###############################################################################################
# $Id: start_game.py,v 1.8 2003/03/09 17:13:31 chakie Exp $
###############################################################################################

import sys
import string
import traceback
import pygame
from pygame.locals import *

import scenario
import properties
import widget
from button           import Button
from image            import Image
from widget_manager   import WidgetManager
from dialog           import *
from messagebox       import Messagebox
from scenario_parser  import ScenarioParser
from scenario_manager import ClientScenarioManager

class StartGame(Dialog):
    """
    This class is used as a dialog for showing some information to the player while the game sets up
    the network connection and receives the scenario information. Shows a centered splash screen
    with a label of information on it.
    """

    def __init__ (self):
        "Creates the dialog."
        # init superclass
        Dialog.__init__ (self, scenario.sdl)

        # set our background to a tiled image
        self.setBackground ( properties.window_background )

        # we need a scenario manager for loading
        self.manager = ClientScenarioManager ()

        # no error yet
        self.error = ()

        # no lines to read yet
        self.linecount = -1
        self.lastratio = -1

        # no progress images yet
        self.progress_images = []
        
        # neither scenario data not config data read
        self.theater_read  = 0
        self.config_read   = 0
        self.scenario_read = 0
        
        # no scenario data yet
        self.buffer = ''
        
        # we want timer events
        self.enableTimer ( 300 )
        

    def createWidgets (self):
        "Creates all widgets for the dialog."

        # create a message
        message = "Waiting for other player to perform setup..."
            
        # create the splash
        self.splash = Image ( properties.dialog_splash, (272, 204) )
        self.wm.register ( self.splash )
 
        # the status label
        self.statuslabel = NormalLabel ( message, (340, 213) )
        self.wm.register ( self.statuslabel )

        # create the cancel button
        self.wm.register ( Button ( properties.path_dialogs + "butt-cancel-moff.png",
                                    properties.path_dialogs + "butt-cancel-mover.png",
                                    (406, 650), {widget.MOUSEBUTTONUP : self.cancel } ) )



    def timer (self):
        """Callback triggered when the dialog has enabled timers and a timer fires. First tries to
        read the configuration from the connection, and if that succeeds further calls will read the
        raw scenario data. If all succeeds the dialog is accepted."""

        # have we read the theater?
        if not self.theater_read:
            # read scenario theater
            return self.readTheater ()

        # do we have lines to read, i.e. have we started to receive the scenario?
        elif not self.config_read:
            # read scenario config
            return self.readConfig ()

        elif not self.scenario_read:
            # read the scenario data instead
            if not self.readScenario ():
                # failed to read the scenario. ugh
                self.state = REJECTED
                return widget.DONE

            # now read the LOS map
            if not self.readLosMap ():
                # failed to read the scenario. ugh
                self.state = REJECTED
                return widget.DONE

        # we're cancelling the dialog
        self.state = ACCEPTED
        return widget.DONE


    def readTheater (self):
        """Reads the theater name from the network connection. This is a single string value that
        tells us what theater the server has selected. If all is ok sets the flag that
        indicates that the theater is now read."""
        
        # wait for the server to send a status
        theater = scenario.connection.readLine ( timeout = 0.1 )

        # did we get a theater
        if not theater:
            # nothing was read, go on
            return

        # clean it up
        theater = theater.strip ()
        
        print "StartGame.readTheater: theater: '%s'" % theater
        print "StartGame.readTheater: *** what now to make it active? ***"

        # store it in the properties
        properties.theater = theater

        
        # theater is now read
        self.theater_read = 1

        return widget.HANDLED


    def readConfig (self):
        """Reads the configuration data from the network connection. If all is ok sets the flag that
        indicates that the configuration is now read."""
        
        # wait for the server to send a status
        status = scenario.connection.readLine ( timeout = 0.01 )

        # did we get a status
        if not status:
            # nothing was read, go on
            return

        # split up the received message into components
        parts = status.split ()

        # precautions. we need  5 parts: "id lines width height name"
        if len ( parts ) != 5:
            # oops, bad data
            Messagebox ( "Bad data received from server!" )

            # repaint and go away
            self.wm.paint (force=1, clear=1)
 
            # we're cancelling the dialog
            self.state = REJECTED
        
            return widget.DONE

        # get all data from the read parts
        scenario.local_player_id = int ( parts[0] )
        scenario.remote_player_name = string.join ( parts [4:] )

        # also the number of lines in the scenario we're about to read
        self.linecount = int ( parts[1] )

        # and the los map size
        self.los_width  = int ( parts[2] )
        self.los_height = int ( parts[3] )
        
        print "StartGame.readConfig: los map size: %d x %d" % (self.los_width, self.los_height)
        
        # configuration is now read
        self.config_read = 1

        return widget.HANDLED


    def readScenario (self):
        """Retrieves a scenario from the server. Sends the id of the scenario to the server and
        reads the XML data the server sends back."""
        
        # set waiting cursor
        self.setWaitCursor ()

        try:
            # let the player know what we do
            self.statuslabel.setText ( 'Downloading scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # now do the raw reading
            buffer = self.manager.readScenario ( scenario.connection, self.linecount, self.progressCallback )

            # let the player know what we do
            self.statuslabel.setText ( 'Building scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # parse the read data
            ScenarioParser ().parseString ( buffer )

            # scenario is read now
            self.scenario_read = 1
 
            # set normal cursor
            self.setNormalCursor ()
           
            # we're done with the dialog
            return 1
        
        except:
            raise
            # serious error
            Messagebox ( "Could not read scenario!" )
            
            # repaint and go away
            self.wm.paint (force=1, clear=1)

            # set normal cursor
            self.setNormalCursor ()
  
            # we're cancelling the dialog
            return 0


    def readLosMap (self):
        """Retrieves the LOS map for a scenario from the server. The new LOS map is assigned to the
        internal map. The user is shown progress using the method progressCallback()."""
        
        # set waiting cursor
        self.setWaitCursor ()

        self.lastratio = 0
        
        try:
            # let the player know what we do
            self.statuslabel.setText ( 'Downloading line of sight data...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # now do the raw reading
            los_map = self.manager.readLosMap ( scenario.connection, self.los_width, self.los_height,
                                                self.progressCallback )

            # let the player know what we do
            self.statuslabel.setText ( 'Building line of sight maps...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # assign the los map
            scenario.map.setLosMap ( los_map )

            # let the player know what we do
            self.statuslabel.setText ( 'Setting up the game...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )
            
            # set normal cursor
            #self.setNormalCursor ()
           
            # we're done with the dialog
            return 1
        
        except:
            raise
            # serious error
            Messagebox ( "Could not read scenario!" )
            
            # repaint and go away
            self.wm.paint (force=1, clear=1)

            # set normal cursor
            self.setNormalCursor ()
  
            # we're cancelling the dialog
            return 0


    def progressCallback (self, current, total):
        """ """
        # loop over all old progress images and deregister them. they may be left from some old
        # progress counter
        for image in self.progress_images:
            self.wm.deRegister ( image )
           
        # clear the images now, we'll get new ones
        self.progress_images = []
        
        # get a ratio of the progress
        ratio = int ( float (current) / total * 32.0 )

        # have we advanced another 10%?
        if ratio > self.lastratio:

            # yep, let the user feel the update
            self.lastratio = ratio

            # create the position
            xpos = 352 + self.lastratio * 10
            ypos = 461

            # create a new part of the progress bar
            progress = Image ( filename=properties.progress_bar_mid, position=(xpos, ypos))
            self.wm.register ( progress )
            
            # add to our progress images
            self.progress_images.append ( progress )
            
            # we must repaint the entire dialog
            self.wm.paint ( clear=0, force=0 )


        
    def readScenario2 (self):
        """Retrieves a scenario from the server. Sends the id of the scenario to the server and
        reads the XML data the server sends back."""

        # optimize away a dot
        connection = scenario.connection

        # set waiting cursor
        self.setWaitCursor ()

        # all progress images. storing them here means that we can later deregister the added images
        progress_images = []
        
        try:
            lastratio = 0
            xpos = 352
            ypos = 461
            
            # create initial progress icon, the leftmost
##             progress = Image ( properties.progress_bar_start, ( 100, 200 ) )
##             self.wm.register ( progress )

            # let the player know what we do
            self.statuslabel.setText ( 'Downloading scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # loop and read the lines
            for index in range ( self.linecount ):
                # read a line
                line = connection.readLine ( timeout = 5 )

                # did we get a line?
                if not line:
                    continue
                
                # add to the internal list
                self.buffer += line

                # get a ratio [0..9] of the progress
                ratio = int ( float (index) / self.linecount * 33.0 )

                # have we advanced another 10%?
                if ratio > lastratio:
                    # yep, let the user feel the update
                    lastratio = ratio

                    # create a new part of the progress bar
                    progress = Image ( filename=properties.progress_bar_mid, position=(xpos, ypos))
                    self.wm.register ( progress )

                    # add to our progress images
                    progress_images.append ( progress )
                    
                    # go onwards
                    xpos += 10
                    
                    # we must repaint the entire dialog
                    self.wm.paint ( clear=0, force=0 )

            # let the player know what we do
            self.statuslabel.setText ( 'Building scenario...' )
            
            # we must repaint so that the label is shown
            self.wm.paint ( clear=1, force=1 )

            # parse the read data
            ScenarioParser ().parseString ( self.buffer )

            # now read the LOS map
            

            # scenario is read now
            self.scenario_read = 1
 
            # set normal cursor
            self.setNormalCursor ()
           
            # we're done with the dialog
            return widget.HANDLED
        
        except:
            raise
            # serious error
            Messagebox ( "Could not read scenario!" )
            
            # repaint and go away
            self.wm.paint (force=1, clear=1)

            # set normal cursor
            self.setNormalCursor ()
  
            # we're cancelling the dialog
            self.state = REJECTED
            return widget.DONE


    def cancel (self, trigger, event):
        """Callback triggered when the user clicks the 'Cancel' button. Simply closes the dialog and
        returns to the main dialog, ignoring any changes."""

        # we're cancelling the dialog
        self.state = REJECTED
        
        return widget.DONE



#  Local Variables:
#  mode: auto-fill
#  fill-column: 100
#  End:
